"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _helperPluginUtils = require("@babel/helper-plugin-utils");
var _core = require("@babel/core");
var _template = _interopRequireDefault(require("./template"));
var _generator = _interopRequireDefault(require("@babel/generator"));
var _generateInitialCoverage = require("./helpers/generate-initial-coverage");
var _generateCanyon = require("./helpers/generate-canyon");
var _package = _interopRequireDefault(require("../package.json"));
function _interopRequireDefault(e) { return e && e.__esModule ? e : { default: e }; }
const canyonTemplate = (0, _core.template)(_template.default["templates/canyon.template.js"]);
const writeCanyonToLocalTemplate = (0, _core.template)(_template.default["templates/write-canyon-to-local-template.js"]);
const injectInPagePopupTemplate = (0, _core.template)(_template.default["templates/inject-in-page-popup.js"]);
// 转换配置，优先级：babel配置 > 环境变量
function convertConfig(config) {
  let defaultCiField = {
    projectID: 'CI_PROJECT_ID',
    buildID: 'CI_JOB_ID',
    commitSha: 'CI_COMMIT_SHA',
    reporter: 'REPORTER',
    dsn: 'DSN',
    branch: 'CI_COMMIT_BRANCH'
  };
  let data = {};
  for (const ciFieldKey in defaultCiField) {
    data[ciFieldKey] = config[ciFieldKey] || process.env[defaultCiField[ciFieldKey]];
  }
  return {
    ...config,
    ...data
  };
}
function newAtob() {
  try {
    return typeof atob === 'function' ? atob : null;
  } catch (e) {
    return null;
  }
}
const newatob = newAtob();
var _default = exports.default = (0, _helperPluginUtils.declare)((api, config) => {
  api.assertVersion(7);
  return {
    visitor: {
      Program: {
        exit(path) {
          // 转换配置
          config = convertConfig(config);
          const __canyon__ = {
            PROJECT_ID: String(config.projectID) || '-',
            BUILD_ID: String(config.buildID) || '-',
            DSN: config.dsn || '-',
            REPORTER: config.reporter || '-',
            INSTRUMENT_CWD: config.instrumentCwd || process.cwd(),
            COMMIT_SHA: config.commitSha || config.sha || '-',
            BRANCH: config.branch || '-',
            REPORT_ID: config.reportID || '-',
            COMPARE_TARGET: config.compareTarget || '-',
            VERSION: _package.default.version || '-'
          };

          // 生成初始覆盖率数据
          const initialCoverageDataForTheCurrentFile = (0, _generateInitialCoverage.generateInitialCoverage)((0, _generator.default)(path.node).code);
          if ((0, _generator.default)(path.node).code.includes('coverageData')) {
            const t = api.types;
            // 遍历 Program 中的所有节点
            path.traverse({
              VariableDeclarator(variablePath) {
                // 检查是否是 coverageData
                if (t.isIdentifier(variablePath.node.id, {
                  name: "coverageData"
                }) && t.isObjectExpression(variablePath.node.init)) {
                  // 查找插桩后的字段
                  const hasInstrumentation = variablePath.node.init.properties.some(prop => t.isIdentifier(prop.key, {
                    name: "_coverageSchema"
                  }) ||
                  // 确保是已插桩的字段
                  t.isIdentifier(prop.key, {
                    name: "s"
                  }) || t.isIdentifier(prop.key, {
                    name: "f"
                  }));
                  if (hasInstrumentation) {
                    // 获取 coverageData 对象的 properties
                    const properties = variablePath.node.init.properties;

                    // 删除 statementMap、fnMap 和 branchMap 属性
                    const keysToRemove = ["statementMap", "fnMap", "branchMap", "inputSourceMap"];
                    keysToRemove.forEach(key => {
                      const index = properties.findIndex(prop => t.isIdentifier(prop.key, {
                        name: key
                      }));
                      if (index !== -1) {
                        properties.splice(index, 1); // 删除属性
                      }
                    });

                    // 增加 sha 字段
                    const shaField = t.objectProperty(t.identifier("sha"),
                    // 键名
                    t.stringLiteral(__canyon__.COMMIT_SHA) // 键值
                    );
                    properties.push(shaField); // 添加新字段
                    // 增加 sha 字段
                    const projectIDField = t.objectProperty(t.identifier("projectID"),
                    // 键名
                    t.stringLiteral(__canyon__.PROJECT_ID) // 键值
                    );
                    properties.push(projectIDField); // 添加新字段
                    // 增加 sha 字段
                    const instrumentCwdField = t.objectProperty(t.identifier("instrumentCwd"),
                    // 键名
                    t.stringLiteral(__canyon__.INSTRUMENT_CWD) // 键值
                    );
                    properties.push(instrumentCwdField); // 添加新字段
                  }
                }
              }
            });
            //   end
          }
        }
      }
    }
  };
});